package app;

import java.awt.event.*;
import java.awt.Image;
import java.awt.Toolkit;
import java.awt.print.*;
import java.io.*;
import java.net.URL;
import java.util.Properties;
import javax.swing.*;

/**
 * Stellt die immer wiederkehrenden Funktion einer Applikation zu Verfgung.<br>
 * <i>(C)opyright 2002 L.H.Hahn</i>
 * @author Dr. Lars H. Hahn
 * @version 3.0
 */

abstract public class Applikation {
  private final static boolean DEBUG = false;

  // Werte mssen durch die abgeleitete Klasse definiert werden:
  public static String name;
  public static String version;
  public static String copyright;
  public static String jarDateiname;
  public static String kuerzel;

  // werden hier spter geladen
  public static Image  image;
  public static Image  icon;

  public static DlgFehlerAusgabe fehlerAusgabe = null;
  private JFrame oberflaeche;
  private MenuHandler menuHandler = new MenuHandler(this);
  private PageFormat seitenFormat = PrinterJob.getPrinterJob().defaultPage();
  private static boolean geaendert = false;
  protected Properties properties = new Properties();
  private DlgOptionen dlgOptionen = null;

  /**
   * einziger Konstruktor
   * @param oberflaeche Hauptfenster der Applikation
   */
  public Applikation(JFrame oberflaeche) {
    this.oberflaeche = oberflaeche;
  }

  /**
   * Liefert das Hauptfenster, das dem Konstruktor bergeben wurde.
   */
  public JFrame getOberflaeche() {
    return oberflaeche;
  }

  /**
   * Liefert den Men-Handler der Applikation.
   * @return Men-Handler der Applikation
   */
  public MenuHandler getMenuHandler() {
    return menuHandler;
  }

  /**
   * Liefert das Seitenformat der Applikation.
   * @return Seitenformat der Applikation
   */
  public PageFormat getSeitenFormat() {
    return seitenFormat;
  }

  /**
   * Initialisiert die Applikation. Muss im Konstruktor der abgeleiteten Klasse
   * aufgerufen werden.
   */
  public void init() {
    if(DEBUG)
      System.out.println("Initialisiere Applikation.");

    // Icon '*-icon.gif' + Bild '*-image.gif' fr Programminfo laden
    icon = bildLaden(kuerzel+"-icon.gif");
    image = bildLaden(kuerzel+"-image.gif");

    // Die Oberflche (=JFrame) wird initialisiert
    oberflaeche.setTitle(copyright);
    oberflaeche.setIconImage(icon);
    oberflaeche.setJMenuBar(menuHandler);
    oberflaeche.setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
    oberflaeche.addWindowListener(new WindowAdapter() {
      public void windowClosing(WindowEvent e) {
        beenden();
      }
    });

    // Die (static) Fehlerausgabe wird erzeugt, aber gleich versteckt
    fehlerAusgabe = new DlgFehlerAusgabe(oberflaeche);
    fehlerAusgabe.pack();
    fehlerAusgabe.setVisible(true);
    fehlerAusgabe.hide();

    // Properties und letzte Datei laden
    propertiesLaden();
    if(properties.containsKey(kuerzel+".dateiname"))
      laden(new File(properties.getProperty(kuerzel+".dateiname")));
    else
      neu();
  }

  /**
   * Ldt Bilder aus dem Verzeichnis <code>images/</code>.
   * Es muss zuerst 'jarDateiname' in der abgeleiteten Klasse gesetzt werden.
   * @param dateiname Dateiname des Bildes (ohne Pfad)
   */
  public Image bildLaden(String dateiname) {
    Image img = null;
    URL url = null;
    String userDir = System.getProperty("user.dir");
    try {
      url = new URL("file:"+userDir+"/images/");
    } catch(java.net.MalformedURLException e) { }
    File datei = new File(userDir, jarDateiname);
    if(datei.exists()) {
      try {
        url = new URL("jar", "", datei.toURL() + "!/images/");
      } catch(java.net.MalformedURLException e) { }
    }
    try {
      img = Toolkit.getDefaultToolkit().getImage(new URL(url, dateiname));
    } catch(java.net.MalformedURLException e) { }
    return img;
  }

  /**
   * Zeigt an, ob die Applikationsdaten gendert wurden. Bevor das Programm
   * verlassen wird oder die Daten berschrieben werden, kann mit
   * abfrageGeaendert() beim Benutzer nachgefragt werden.
   * @param b <code>true</code> - Applikation wurde gendert,
   *          <code>false</code> - Daten sind gespeichert
   */
  static public void geaendert(boolean b) {
    geaendert = b;
    if(DEBUG)
      System.out.println("Applikation gendert ->"+b);
  }

  /**
   * berprft, ob die Applikationsdaten gendert wurden.
   * @return <code>true</code> - Applikation wurde gendert,
   *         <code>false</code> - Daten sind gespeichert
   */
  protected boolean abfrageGeaendert() {
    if(geaendert) {
      int n = JOptionPane.showConfirmDialog(oberflaeche,
        "Die Daten wurden gendert.\nSoll zunchst gespeichert werden?");
      switch(n) {
        case JOptionPane.CANCEL_OPTION: return false;
        case JOptionPane.NO_OPTION: return true;
        case JOptionPane.OK_OPTION: speichern(); break;
      }
      if(geaendert) // Speichern war nicht erfolgreich!
        return false;
    }
    return true;
  }

  /**
   * Druckt oder zeigt eine 'PrintablePane' an.
   * @param ueberschrift erscheint in der Fenster-Titelzeile oder als Name des
   *                     Druckjobs
   * @param pane         Seite, die angezeigt oder gedruckt werden soll
   */
  protected void anzeigenOderDrucken(String ueberschrift, AbstractPrintablePane pane) {
    boolean ausgabeDrucker = menuHandler.getState("X_DRUCKER");
    String font = properties.getProperty(kuerzel+".opt.font", "SansSerif");
    int punkt = Integer.parseInt(properties.getProperty(kuerzel+".opt.punkt", "12"));
    pane.setFont(font, punkt);
    if(ausgabeDrucker) {
      PrinterJob job = PrinterJob.getPrinterJob();
      job.setJobName(version+" - "+ueberschrift);
      job.setPrintable(pane, seitenFormat);
      if(job.printDialog()) {
        try {
          job.print();
        }
        catch(PrinterException e) {
          fehlerAusgabe.println("-E- Fehler beim Drucken!");
        }
      }
    }
    else {
      JFrame frame = new JFrame(ueberschrift);
      frame.setIconImage(icon);
      frame.getContentPane().add(new JScrollPane(pane));
      frame.pack();
      frame.setVisible(true);
    }
  }

  /**
   * Ldt die individuellen Programmeigenschaften aus der Datei ".<i>kuerzel</i>".
   */
  private void propertiesLaden() {
    String userHome = System.getProperty("user.home");
    properties.setProperty(kuerzel+".ordner", userHome); // Default-Wert Arbeitsordner
    File datei = new File(userHome,"."+kuerzel);
    if(datei.exists()) {
      try {
        FileInputStream fis = new FileInputStream(datei);
        properties.load(fis);
        fis.close();
        if(DEBUG)
          properties.list(System.out);
      }
      catch(Exception e) {
        fehlerAusgabe.println("-I- Keine Properties geladen.");
      }
    }
  }

  /**
   * Speichert die individuellen Programmeigenschaften in die Datei ".<i>kuerzel</i>".
   */
  private void propertiesSpeichern() {
    try {
      String userHome = System.getProperty("user.home");
      File datei = new File(userHome, "."+kuerzel);
      FileOutputStream fos = new FileOutputStream(datei);
      properties.store(fos, "Properties: "+version);
      fos.close();
    }
    catch(Exception e) {
      fehlerAusgabe.println("-W- Properties konnten nicht ins Home-Verzeichnis geschrieben werden.");
    }
  }

 /**
  * Wird von Men-Handler bei Auswahl von '<b>Datei/Neu</b>' aufgerufen.
  * @return <code>false</code> - Benutzer hat 'Abruch' gewhlt.
  */
  public boolean neu() {
    if(abfrageGeaendert()) {
      properties.remove(kuerzel+".dateiname");
      geaendert(false);
      return true;
    }
    return false;
  }

  /**
   * Wird von Men-Handler bei Auswahl von '<b>Datei/Laden</b>' aufgerufen.
   */
  public void laden() {
    if(neu()) {
      JFileChooser dateidialog = new JFileChooser();
      dateidialog.setCurrentDirectory(new File(properties.getProperty(kuerzel+".ordner")));
      if(dateidialog.showOpenDialog(oberflaeche) == JFileChooser.APPROVE_OPTION)
        laden(dateidialog.getSelectedFile());
    }
  }

  /**
   * Enthlt die eigentliche Laderoutine.
   */
  abstract protected void laden(DataInputStream in)
    throws IOException;

  private void laden(File datei) {
    try {
      FileInputStream fis = new FileInputStream(datei);
      DataInputStream in = new DataInputStream(fis);
      laden(in);
      fis.close();
      properties.setProperty(kuerzel+".dateiname", datei.getPath());
      geaendert(false);
    }
    catch(FileNotFoundException e1) {
      fehlerAusgabe.println("-E- "+datei.getPath()+" nicht gefunden!");
    }
    catch(IOException e2) {
      fehlerAusgabe.println("-E- Fehler beim Laden: "+datei.getPath());
    }
  }

  /**
   * Wird von Men-Handler bei Auswahl von '<b>Datei/Speichern</b>' aufgerufen.
   */
  public void speichern() {
    String dateiname = properties.getProperty(kuerzel+".dateiname");
    if(DEBUG)
      System.out.println("Speichern: Dateiname="+dateiname);
    if(dateiname == null)
      speichernUnter();
    else
      speichernUnter(new File(dateiname));
  }

  /**
   * Enthlt die eigentliche Routine zum Speichern.
   */
  abstract protected void speichern(DataOutputStream out)
    throws IOException;

  private void speichernUnter(File datei) {
    try {
      if(!datei.getName().toLowerCase().endsWith("."+kuerzel)) {
        String name = datei.getAbsolutePath()+"."+kuerzel;
        datei = new File(name);
        fehlerAusgabe.println("Dateiname gendert zu "+name);
      }
      FileOutputStream fos = new FileOutputStream(datei);
      DataOutputStream out = new DataOutputStream(fos);
      speichern(out);
      out.flush();
      fos.close();
      properties.setProperty(kuerzel+".dateiname", datei.getPath());
      geaendert(false);
    }
    catch(FileNotFoundException e1) {
      fehlerAusgabe.println("-E- "+datei.getPath()+" nicht gefunden!");
    }
    catch(IOException e2) {
      fehlerAusgabe.println("-E- Fehler beim Speichern: "+datei.getPath());
    }
  }

  /**
   * Wird von Men-Handler bei Auswahl von '<b>Datei/Speichern unter</b>' aufgerufen.
   */
  public void speichernUnter() {
    JFileChooser dateidialog = new JFileChooser();
    dateidialog.setCurrentDirectory(new File(properties.getProperty(kuerzel+".ordner")));
    if(dateidialog.showSaveDialog(oberflaeche) == JFileChooser.APPROVE_OPTION)
      speichernUnter(dateidialog.getSelectedFile());
  }

  /**
   * Wird von Men-Handler bei Auswahl von '<b>Datei/Seite einrichten</b>' aufgerufen.
   */
  public void seiteEinrichten() {
    seitenFormat = PrinterJob.getPrinterJob().pageDialog(seitenFormat);
  }

  /**
   * Wird von Men-Handler bei Auswahl von '<b>Datei/Beenden unter</b>' aufgerufen.
   */
  public void beenden() {
    if(abfrageGeaendert()) {
      propertiesSpeichern();
      System.exit(0);
    }
  }

  /**
   * Liefert den Dialog zur Auswahl der Optionen.
   */
  protected DlgOptionen getDlgOptionen() {
    return dlgOptionen;
  }

  /**
   * Initialisiert die Optionen von <code>dlgOptionen</code>.
   * Es werden die beiden Optionen "Ausgabe auf dem Drucker" und "Arbeitsordner"
   * definiert. Die Funktion muss zum Hinzufgen weiterer Optionen berladen
   * werden (benutze: addCheckBox und addTextField).
   * @see DlgOptionen
   */
  protected void initOptionen() {
    dlgOptionen = new DlgOptionen(oberflaeche);
    dlgOptionen.addCheckBox("X_DRUCKER", "Ausgabe auf den Drucker:", menuHandler.getState("X_DRUCKER"));
    dlgOptionen.addTextField("ORDNER", "Arbeitsordner:", properties.getProperty(kuerzel+".ordner"));
    String[] fontAuswahl = {"SansSerif", "Serif", "Monospaced"};
    String font = properties.getProperty(kuerzel+".opt.font", "SansSerif");
    dlgOptionen.addComboBox("FONT", "Zeichensatz:", fontAuswahl, font);
    String punkt = properties.getProperty(kuerzel+".opt.punkt", "12");
    dlgOptionen.addTextField("PUNKT", "Gre des Zeichensatzes (Punkt):", punkt);
  }

  /**
   * bertrgt die Optionen von <code>dlgOptionen</code> nach Schlieen des Dialogs.
   * Es werden die beiden Optionen "Ausgabe auf dem Drucker" und "Arbeitsordner"
   * betragen. Die Funktion muss zum Hinzufgen weiterer Optionen berladen
   * werden (benutze: getWert).
   * @see DlgOptionen
   */
  protected void uebertrageOptionen() {
    menuHandler.setState("X_DRUCKER", ((Boolean)dlgOptionen.getWert("X_DRUCKER")).booleanValue());
    properties.setProperty(kuerzel+".ordner", (String)dlgOptionen.getWert("ORDNER"));
    String font = (String)dlgOptionen.getWert("FONT");
    properties.setProperty(kuerzel+".opt.font", font);
    String punkt = (String)dlgOptionen.getWert("PUNKT");
    properties.setProperty(kuerzel+".opt.punkt", punkt);
  }

  /**
   * Wird von Men-Handler bei Auswahl von '<b>Extras/Optionen</b>' aufgerufen.
   */
  public void optionen() {
    initOptionen();
    if(dlgOptionen == null)
      fehlerAusgabe.println("-W- Keine Optionen definiert.");
    else
      dlgOptionen.showDialog();
    uebertrageOptionen();
  }

  /**
   * Wird von Men-Handler bei Auswahl von '<b>Hilfe/Inhalt</b>' aufgerufen.
   * Es wird standardmig die Datei "html/help.html" angezeigt.
   */
  public void hilfeInhalt() {
    try {
      URL url = null;
      String userDir = System.getProperty("user.dir");
      url = new URL("file:"+userDir+"/html/");
      File datei = new File(userDir, jarDateiname);
      if(datei.exists()) {
        url = new URL("jar", "", datei.toURL() + "!/html/");
      }
      DlgHilfe dlg = new DlgHilfe(oberflaeche);
      dlg.zeigeURL(new URL(url+"help.html"));
      dlg.pack();
      dlg.setVisible(true);
    } catch(java.net.MalformedURLException e) { }
  }

  /**
   * Wird von Men-Handler bei Auswahl von '<b>Hilfe/Programm-Info</b>' aufgerufen.
   */
  public void programmInfo() {
    DlgInfo dlg = new DlgInfo(this);
    dlg.pack();
    dlg.setVisible(true);
  }

}