package app;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.print.*;
import java.util.StringTokenizer;

/**
 * Zeigt Tabellen an mit der Option sie auszudrucken. Die Trennung der
 * Spalten erfolgt per Tabulator '\\t'.<br>
 * <i>(C)opyright 2002 L.H.Hahn</i>
 * @author Dr. Lars H. Hahn
 * @version 3.1
 */

public class MultiPageViewer
  extends JDialog {

  private int[] tabs;
  private int aktuelleSeite = 1;
  private int letzteSeite;
  private int zeilenProSeite;
  private String[] text;
  private String ueberschrift;
  private final PageFormat seitenFormat;
  private Font font;
  private Font kursiv;
  private final PagePane pagePane;
  private final JPanel buttonPane = new JPanel();
  private final JButton next = new JButton("Nchste Seite");
  private final JButton prev = new JButton("Vorherige Seite");
  private final JButton drucken = new JButton("Drucken");
  private final JButton ende = new JButton("Beenden");

  /**
   * Einziger Konstruktor.
   * @param parent Eltern-Fenster
   * @param uberschrift Text, der im Dialog-Titel und in der Fuzeile des
   *        Dokumentes erscheint.
   * @param text Tabelle, die ausgegeben werden soll.
   * @param seitenFormat definiert das Seitenformat der Ausgabe
   */
  public MultiPageViewer(JFrame parent, String ueberschrift, String[] text, PageFormat seitenFormat) {
    super(parent, ueberschrift, true);
    this.text = text;
    this.ueberschrift = ueberschrift;
    this.seitenFormat = seitenFormat;
    pagePane = new PagePane();
    setFont("Helvetica", 12);
    buttonPane.add(next);
    buttonPane.add(prev);
    buttonPane.add(drucken);
    buttonPane.add(ende);
    next.addActionListener(new ActionListener(){
      public void actionPerformed(ActionEvent e) {
        if(aktuelleSeite < letzteSeite) {
          aktuelleSeite++;
          pagePane.repaint();
        }
      }
    });
    prev.addActionListener(new ActionListener(){
      public void actionPerformed(ActionEvent e) {
        if(aktuelleSeite > 1) {
          aktuelleSeite--;
          pagePane.repaint();
        }
      }
    });
    drucken.addActionListener(new ActionListener(){
      public void actionPerformed(ActionEvent e) {
        drucken();
      }
    });
    ende.addActionListener(new ActionListener(){
      public void actionPerformed(ActionEvent e) {
        setVisible(false);
      }
    });
    JScrollPane scrollPane = new JScrollPane(pagePane);
    scrollPane.setPreferredSize(new Dimension((int)seitenFormat.getWidth(), 500));
    this.getContentPane().add(scrollPane, BorderLayout.CENTER);
    this.getContentPane().add(buttonPane, BorderLayout.SOUTH);
    double[] relTabs = {0.0, 10.0, 20.0, 30.0, 40.0, 50.0, 60.0, 70.0, 80.0, 90.0};
    this.setTabs(relTabs);
  }

  /**
   * Zeigt den Dialog auf dem Bildschirm an, wenn er noch nicht sichtbar ist.
   */
  public void showDialog() {
    pack();
    setVisible(true);
  }

  /**
   * Verndert die berschrift, die im Dialog-Titel und in der Fuzeile des
   * Dokumentes erscheint.
   * @param ueberschrift neue berschrift
   */
  public void setUeberschrift(String ueberschrift) {
    this.ueberschrift = ueberschrift;
    this.setTitle(ueberschrift);
  }

  /**
   * Ermglicht im oberen Teil des Dialogs eine beliebige Pane anzuzeigen.
   * @param auswahlPane Pane zur Auswahl spezieller Parameter
   */
  public void setAuswahlPane(JPanel auswahlPane) {
    this.getContentPane().add(auswahlPane, BorderLayout.NORTH);
  }

  /**
   * ndert die Tabelle, die angezeigt wird.
   * @param text Tabelle, die ausgegeben werden soll.
   */
  public void setText(String[] text) {
    this.text = text;
    aktuelleSeite = 1;
    seitenAnzahlBerechnen();
    pagePane.repaint();
  }

  /**
   * Setzt die Tabulatoren der Tabelle.
   * @param relTabs relative Tabulatoren 0.0 bis 100.0
   */
  public void setTabs(double[] relTabs) {
    int anzahl = relTabs.length;
    tabs = new int[anzahl];
    for(int i=0; i<anzahl; i++)
      tabs[i] = (int)(seitenFormat.getImageableWidth() * relTabs[i] / 100.0 + seitenFormat.getImageableX());
  }

  /**
   * Setzt den Zeichensatz der Tabelle.
   * @param name Name des Zeichensatzes
   * @param punkt Gre des Zeichensatzes (in Punkt)
   */
  public void setFont(String name, int punkt) {
    font = new Font(name, Font.PLAIN, punkt);
    kursiv = new Font(name, Font.ITALIC, punkt);
    seitenAnzahlBerechnen();
  }

  private void seitenAnzahlBerechnen() {
    zeilenProSeite = (int)(seitenFormat.getImageableHeight() / pagePane.getFontMetrics(font).getHeight()) - 2;
    letzteSeite = text.length / zeilenProSeite;
    if((text.length - letzteSeite * zeilenProSeite) > 0)
      letzteSeite++;
  }

  private void drucken() {
    PrinterJob job = PrinterJob.getPrinterJob();
    job.setJobName(ueberschrift);
    job.setPrintable(pagePane, seitenFormat);
    if(job.printDialog()) {
      try {
        job.print();
      }
      catch(PrinterException e) {
        Applikation.fehlerAusgabe.println("-E- Fehler beim Drucken!");
      }
    }
  }

  private class PagePane
    extends JPanel
    implements Printable {

    public PagePane() {
      this.setPreferredSize(new Dimension((int)seitenFormat.getWidth(), (int)seitenFormat.getHeight()));
    }

    public void paintComponent(Graphics g) {
      setBackground(Color.white);
      super.paintComponent(g); //paint background
      print(g, seitenFormat, aktuelleSeite-1);
    }

    public int print(Graphics g, PageFormat pageFormat, int seite) {
      if(seite >= letzteSeite)
        return NO_SUCH_PAGE;
      g.setFont(font);
      int y = (int)(pageFormat.getImageableY() + g.getFontMetrics().getHeight() + 0.5D);
      for(int i=0; i<zeilenProSeite; i++) {
        int zeile = seite*zeilenProSeite+i;
        if(zeile < text.length) {
          int c = 0;
          StringTokenizer st = new StringTokenizer(text[zeile], "\t");
          while(st.hasMoreTokens()) {
            String wort = st.nextToken();
            int zellenBreite;
            if(c == (tabs.length-1))
              zellenBreite = (int)(seitenFormat.getImageableWidth() + seitenFormat.getImageableX()) - tabs[c];
            else
              zellenBreite = tabs[c+1] - tabs[c];
            while(g.getFontMetrics().stringWidth(wort) > zellenBreite)
              wort = wort.substring(0, wort.length()-1);
            g.drawString(wort, tabs[c++], y);
            if(c == tabs.length)
              c--;
          }
        }
        y += g.getFontMetrics().getHeight();
      }
      // Fuzeile:
      g.setFont(kursiv);
      y += g.getFontMetrics().getHeight();
      int x = (int)(pageFormat.getImageableX() + 0.5D);
      g.drawString(ueberschrift, x, y);
      String seitenzahl = ""+(seite+1)+"/"+letzteSeite;
      x += (int)(pageFormat.getImageableWidth() - g.getFontMetrics().stringWidth(seitenzahl) + 0.5D);
      g.drawString(seitenzahl, x, y);
      return PAGE_EXISTS;
    }

  }

}